import { ɵɵdefineInjectable, Injectable, EventEmitter, Directive, ElementRef, NgZone, Input, Output, NgModule, Optional, SkipSelf } from '@angular/core';
import { Chart, Title, Tooltip, Filler, Legend, LineController, LineElement, PointElement, LinearScale, CategoryScale, BarController, BarElement, DoughnutController, ArcElement, RadarController, RadialLinearScale, PieController, PolarAreaController, BubbleController, ScatterController, TimeSeriesScale, defaults } from 'chart.js';
import { assign, merge } from 'lodash-es';
import { BehaviorSubject } from 'rxjs';
import { distinctUntilChanged } from 'rxjs/operators';

class ThemeService {
    constructor() {
        this.colorschemesOptions = new BehaviorSubject(null);
    }
    setColorschemesOptions(options) {
        this.pColorschemesOptions = options;
        this.colorschemesOptions.next(options);
    }
    getColorschemesOptions() {
        return this.pColorschemesOptions;
    }
}
ThemeService.ɵprov = ɵɵdefineInjectable({ factory: function ThemeService_Factory() { return new ThemeService(); }, token: ThemeService, providedIn: "root" });
ThemeService.decorators = [
    { type: Injectable, args: [{
                providedIn: 'root'
            },] }
];
ThemeService.ctorParameters = () => [];

class BaseChartDirective {
    constructor(element, zone, themeService) {
        this.element = element;
        this.zone = zone;
        this.themeService = themeService;
        this.chartClick = new EventEmitter();
        this.chartHover = new EventEmitter();
        this.subs = [];
        this.ctx = element.nativeElement.getContext('2d');
        this.subs.push(this.themeService.colorschemesOptions
            .pipe(distinctUntilChanged())
            .subscribe(r => this.themeChanged(r)));
    }
    ngOnChanges(changes) {
        const requireRender = ['type'];
        if (requireRender.some(key => changes.hasOwnProperty(key))) {
            this.render();
        }
        else {
            this.update();
        }
    }
    ngOnDestroy() {
        if (this.chart) {
            this.chart.destroy();
            this.chart = void 0;
        }
        this.subs.forEach(s => s.unsubscribe());
    }
    render() {
        if (this.chart) {
            this.chart.destroy();
        }
        if (this.ctx) {
            return this.chart = new Chart(this.ctx, this.getChartConfiguration());
        }
    }
    update(duration) {
        if (this.chart) {
            this.zone.runOutsideAngular(() => this.chart.update(duration));
        }
    }
    hideDataset(index, hidden) {
        if (this.chart) {
            this.chart.getDatasetMeta(index).hidden = hidden;
            this.update();
        }
    }
    isDatasetHidden(index) {
        return this.chart.getDatasetMeta(index).hidden;
    }
    toBase64Image() {
        return this.chart.toBase64Image();
    }
    themeChanged(options) {
        this.themeOverrides = options;
        if (this.chart) {
            assign(this.chart.config.options, this.getChartOptions());
            this.update();
        }
    }
    getChartOptions() {
        return merge({
            onHover: (event, active) => {
                if (active && !active.length) {
                    return;
                }
                this.chartHover.emit({ event, active });
            },
            onClick: (event, active) => {
                this.chartClick.emit({ event, active });
            }
        }, this.themeOverrides, this.options, {
            legend: {
                display: this.legend
            }
        });
    }
    getChartConfiguration() {
        return merge({
            type: this.type,
            data: this.data,
            plugins: this.plugins,
            options: this.getChartOptions()
        }, {
            data: {
                labels: this.labels,
                datasets: this.datasets
            }
        });
    }
}
BaseChartDirective.decorators = [
    { type: Directive, args: [{
                // tslint:disable-next-line:directive-selector
                selector: 'canvas[baseChart]',
                exportAs: 'base-chart'
            },] }
];
BaseChartDirective.ctorParameters = () => [
    { type: ElementRef },
    { type: NgZone },
    { type: ThemeService }
];
BaseChartDirective.propDecorators = {
    type: [{ type: Input }],
    legend: [{ type: Input }],
    data: [{ type: Input }],
    options: [{ type: Input }],
    plugins: [{ type: Input }],
    labels: [{ type: Input }],
    datasets: [{ type: Input }],
    chartClick: [{ type: Output }],
    chartHover: [{ type: Output }]
};

const baseColors = [
    [255, 99, 132],
    [54, 162, 235],
    [255, 206, 86],
    [231, 233, 237],
    [75, 192, 192],
    [151, 187, 205],
    [220, 220, 220],
    [247, 70, 74],
    [70, 191, 189],
    [253, 180, 92],
    [148, 159, 177],
    [77, 83, 96]
];

const ɵ0 = context => rgba(generateColor(context.datasetIndex), 0.4), ɵ1 = context => rgba(generateColor(context.datasetIndex), 1), ɵ2 = context => rgba(generateColor(context.datasetIndex), 1), ɵ3 = context => rgba(generateColor(context.datasetIndex), 0.8), ɵ4 = context => rgba(generateColor(context.datasetIndex), 0.6), ɵ5 = context => rgba(generateColor(context.datasetIndex), 1), ɵ6 = context => rgba(generateColor(context.datasetIndex), 0.8), ɵ7 = context => rgba(generateColor(context.datasetIndex), 1), ɵ8 = context => rgba(generateColor(context.dataIndex), 0.6), ɵ9 = context => rgba(generateColor(context.dataIndex), 1), ɵ10 = context => rgba(generateColor(context.dataIndex), 1), ɵ11 = context => rgba(generateColor(context.dataIndex), 0.6), ɵ12 = context => rgba(generateColor(context.dataIndex), 1), ɵ13 = context => rgba(generateColor(context.dataIndex), 0.8), ɵ14 = context => rgba(generateColor(context.dataIndex), 1);
const builtInDefaults = {
    datasets: {
        line: {
            backgroundColor: ɵ0,
            borderColor: ɵ1,
            pointBackgroundColor: ɵ2,
            pointBorderColor: '#fff',
            pointHoverBackgroundColor: '#fff',
            pointHoverBorderColor: ɵ3
        },
        bar: {
            backgroundColor: ɵ4,
            borderColor: ɵ5,
            hoverBackgroundColor: ɵ6,
            hoverBorderColor: ɵ7
        },
        get radar() {
            return this.line;
        },
        doughnut: {
            backgroundColor: ɵ8,
            borderColor: '#fff',
            hoverBackgroundColor: ɵ9,
            hoverBorderColor: ɵ10
        },
        get pie() {
            return this.doughnut;
        },
        polarArea: {
            backgroundColor: ɵ11,
            borderColor: ɵ12,
            hoverBackgroundColor: ɵ13,
            hoverBorderColor: ɵ14
        },
        get bubble() {
            return this.doughnut;
        },
        get scatter() {
            return this.doughnut;
        },
        get area() {
            return this.polarArea;
        }
    }
};
function rgba(colour, alpha) {
    return 'rgba(' + colour.concat(alpha).join(',') + ')';
}
function getRandomInt(min, max) {
    return Math.floor(Math.random() * (max - min + 1)) + min;
}
function getRandomColor() {
    return [getRandomInt(0, 255), getRandomInt(0, 255), getRandomInt(0, 255)];
}
/**
 * Generate colors
 */
function generateColor(index = 0) {
    return baseColors[index] || getRandomColor();
}

Chart.register(Title, Tooltip, Filler, Legend, LineController, LineElement, PointElement, LinearScale, CategoryScale, BarController, BarElement, DoughnutController, ArcElement, RadarController, RadialLinearScale, PieController, PolarAreaController, BubbleController, ScatterController, TimeSeriesScale);
class ChartsModule {
    constructor(parentModule) {
        if (parentModule) {
            throw new Error('ChartsModule is already loaded. Import it in the AppModule only');
        }
    }
    static forRoot(config) {
        Chart.register(config.plugins || []);
        const ngChartsDefaults = merge(builtInDefaults, config.defaults);
        defaults.set(ngChartsDefaults);
        return {
            ngModule: ChartsModule,
            providers: [ThemeService]
        };
    }
}
ChartsModule.decorators = [
    { type: NgModule, args: [{
                imports: [],
                declarations: [BaseChartDirective],
                exports: [BaseChartDirective]
            },] }
];
ChartsModule.ctorParameters = () => [
    { type: ChartsModule, decorators: [{ type: Optional }, { type: SkipSelf }] }
];

/*
 * Public API Surface of ng2-charts
 */

/**
 * Generated bundle index. Do not edit.
 */

export { BaseChartDirective, ChartsModule, ThemeService, baseColors };
//# sourceMappingURL=ng2-charts.js.map
